"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Moment = __importStar(require("moment"));
const moment_range_1 = require("moment-range");
const utils_general_1 = require("@energyweb/utils-general");
const PreciseProofEntity_1 = require("./PreciseProofEntity");
class Entity extends PreciseProofEntity_1.PreciseProofEntity {
    constructor(id, configuration, isPrivate = false) {
        super(id, configuration);
        this.isPrivate = isPrivate;
        this.initialized = false;
    }
    async sync() {
        const { issuer } = this.configuration.blockchainProperties;
        const issueRequest = await issuer.getCertificationRequest(this.id);
        const decodedData = await issuer.decodeData(issueRequest.data);
        this.owner = issueRequest.owner;
        this.fromTime = Number(decodedData['0']);
        this.toTime = Number(decodedData['1']);
        this.deviceId = decodedData['2'];
        this.approved = issueRequest.approved;
        this.revoked = issueRequest.revoked;
        const newCertificationRequestEvent = await issuer.getAllNewCertificationRequestEvents({
            filter: { _id: this.id }
        });
        const creationBlock = await this.configuration.blockchainProperties.web3.eth.getBlock(newCertificationRequestEvent[0].blockNumber);
        this.created = Number(creationBlock.timestamp);
        const offChainData = await this.configuration.offChainDataSource.certificateClient.getCertificationRequestData(this.id);
        this.energy = offChainData.energy;
        this.files = offChainData.files;
        this.initialized = true;
        if (this.configuration.logger) {
            this.configuration.logger.verbose(`CertificationRequest ${this.id} synced`);
        }
        return this;
    }
    async approve() {
        const validityData = this.configuration.blockchainProperties.web3.eth.abi.encodeFunctionCall({
            name: 'isRequestValid',
            type: 'function',
            inputs: [
                {
                    type: 'uint256',
                    name: '_requestId'
                }
            ]
        }, [this.id.toString()]);
        let approveTx;
        const { issuer } = this.configuration.blockchainProperties;
        if (this.isPrivate) {
            const commitment = {
                [this.owner]: this.energy
            };
            const { rootHash } = this.generateAndAddProofs(commitment);
            approveTx = await issuer.approveCertificationRequestPrivate(this.id, rootHash, validityData, utils_general_1.Configuration.getAccount(this.configuration));
        }
        else {
            approveTx = await issuer.approveCertificationRequest(this.id, this.energy, validityData, utils_general_1.Configuration.getAccount(this.configuration));
        }
        return this.configuration.blockchainProperties.web3.utils.hexToNumber(approveTx.logs[1].topics[1]);
    }
    async revoke() {
        await this.configuration.blockchainProperties.issuer.revokeRequest(this.id, utils_general_1.Configuration.getAccount(this.configuration));
    }
    async requestMigrateToPublic(value) {
        if (!this.isPrivate) {
            throw new Error('Certificate is already public.');
        }
        const { issuer } = this.configuration.blockchainProperties;
        const commitment = {
            [this.configuration.blockchainProperties.activeUser.address]: value
        };
        const { rootHash } = this.generateAndAddProofs(commitment);
        return issuer.requestMigrateToPublic(this.id, rootHash);
    }
}
exports.Entity = Entity;
exports.createCertificationRequest = async (fromTime, toTime, energy, deviceId, configuration, files, isVolumePrivate = false, forAddress) => {
    const request = new Entity(null, configuration, isVolumePrivate);
    const { issuer } = configuration.blockchainProperties;
    await validateGenerationPeriod(fromTime, toTime, deviceId, configuration, isVolumePrivate);
    const data = await issuer.encodeData(fromTime, toTime, deviceId);
    const { logs } = await (forAddress
        ? issuer.requestCertificationFor(data, forAddress, isVolumePrivate, utils_general_1.Configuration.getAccount(configuration))
        : issuer.requestCertification(data, isVolumePrivate, utils_general_1.Configuration.getAccount(configuration)));
    request.id = configuration.blockchainProperties.web3.utils.hexToNumber(logs[0].topics[2]);
    await configuration.offChainDataSource.certificateClient.updateCertificationRequestData(request.id, { energy, files });
    if (configuration.logger) {
        configuration.logger.info(`CertificationRequest ${request.id} created`);
    }
    return request.sync();
};
const validateGenerationPeriod = async (fromTime, toTime, deviceId, configuration, isPrivate) => {
    const { issuer } = configuration.blockchainProperties;
    const moment = moment_range_1.extendMoment(Moment);
    const unix = (timestamp) => moment.unix(timestamp);
    const certificationRequestIds = await issuer.getCertificationRequestsForDevice(deviceId);
    const generationTimeRange = moment.range(unix(fromTime), unix(toTime));
    for (const id of certificationRequestIds) {
        const certificationRequest = await new Entity(id, configuration, isPrivate).sync();
        if (certificationRequest.revoked) {
            continue;
        }
        const certificationRequestGenerationRange = moment.range(unix(certificationRequest.fromTime), unix(certificationRequest.toTime));
        if (generationTimeRange.overlaps(certificationRequestGenerationRange)) {
            throw new Error(`Generation period ` +
                `${unix(fromTime).format()} - ${unix(toTime).format()}` +
                ` overlaps with the time period of ` +
                `${unix(certificationRequest.fromTime).format()} - ${unix(certificationRequest.toTime).format()}` +
                ` of request ${certificationRequest.id}.`);
        }
    }
    return true;
};
async function getAllCertificationRequests(configuration) {
    const { issuer } = configuration.blockchainProperties;
    const totalRequests = await issuer.totalRequests();
    const certificationRequestPromises = Array(Number(totalRequests))
        .fill(null)
        .map(async (item, index) => new Entity(index + 1, configuration).sync());
    return Promise.all(certificationRequestPromises);
}
exports.getAllCertificationRequests = getAllCertificationRequests;
//# sourceMappingURL=CertificationRequest.js.map